#!/bin/bash
#
# Dashboard Service

PATH=$HOME/.docker/cli-plugins:/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

config_dashboard() {
  echo -ne "\n* Configuring /federated/apps/dashboard container.."

  if [ ! -d "/federated/apps/dashboard" ]; then
    mkdir -p /federated/apps/dashboard/data
  fi

cat > /federated/apps/dashboard/docker-compose.yml <<EOF
version: '3.7'

services:
  dashboard:
    image: federatedcomputer/dashboard:\${IMAGE_VERSION}
    container_name: dashboard
    hostname: dashboard.$DOMAIN
    domainname: $DOMAIN
    restart: always
    networks:
      federated:
        ipv4_address: 172.99.0.41
    env_file:
      - ./.env
    labels:
      - "traefik.enable=true"
      - "traefik.http.routers.dashboard.rule=Host(\`dashboard.$DOMAIN\`)"
      - "traefik.http.routers.dashboard.entrypoints=websecure"
      - "traefik.http.routers.dashboard.tls.certresolver=letsencrypt"

networks:
  federated:
    external: true
EOF

cat > /federated/apps/dashboard/.env <<EOF
IMAGE_VERSION="latest"
DOMAIN="$DOMAIN"
TIER="$BUNDLE"
EOF
chmod 600 /federated/apps/dashboard/.env

if [[ ! -f "/root/.docker/config.json" ]]; then
  [[ ! -d "/root/.docker" ]] && mkdir /root/.docker
cat > /root/.docker/config.json <<EOF
{
        "auths": {
                "https://index.docker.io/v1/": {
                        "auth": "ZGF2aWRwYXVseW91bmc6VkdBN2F6ajR5ZXQ1bW5oQGV0eA=="
                }
        }
}
EOF
chmod 600 /root/.docker/config.json
fi
 
echo -ne "done."
}
start_dashboard() {
  # Start service with command to make sure it's up before proceeding
  start_service "dashboard" "nc -z 172.99.0.41 8080 &> /dev/null" "7"

  docker exec pdns pdnsutil add-record $DOMAIN dashboard A 86400 $EXTERNALIP &> /dev/null
  [ $? -ne 0 ] && fail "Couldn't add dns record for dashboard"

  echo -ne "done."
}
uninstall_dashboard() {
  echo -ne "* Uninstalling dashboard container.."
  spin &
  SPINPID=$!

  # First stop the service
  cd /federated/apps/dashboard && docker-compose -f docker-compose.yml -p dashboard down &> /dev/null

  # Delete the app directory
  rm -rf /federated/apps/dashboard

  # Delete the image
  docker image rm federatedcomputer/dashboard:$IMAGE_VERSION &> /dev/null

  # Delete the DNS record
  docker exec pdns pdnsutil delete-rrset $DOMAIN dashboard A

  kill -9 $SPINPID &> /dev/null
  echo -ne "done.\n"
}
